@@ -1,5 +1,7 @@
 from enstop.plsa import PLSA
 from enstop.streamed_plsa import StreamedPLSA
 from enstop.block_parallel_plsa import BlockParallelPLSA
+from enstop.distributed_plsa import DistributedPLSA
+from enstop.cuda_plsa import GPUPLSA
 from enstop.enstop_ import EnsembleTopics
 from enstop.utils import log_lift, mean_log_lift, coherence, mean_coherence
