@@ -12,7 +12,7 @@ from enstop.block_parallel_plsa import (
     plsa_partial_m_step_on_a_block,
 )
 
-from dask import delayed
+from dask import delayed, compute, optimize, persist
 import dask.array as da
 
 
@@ -68,8 +68,8 @@ def plsa_em_step_dask(
     m = p_w_given_z.shape[1]
     k = p_z_given_d.shape[1]
 
-    result_p_w_given_z = [[] for i in range(n_d_blocks)]
-    result_p_z_given_d = [[] for i in range(n_w_blocks)]
+    result_p_w_given_z = [[] for i in range(n_w_blocks)]
+    result_p_z_given_d = [[] for i in range(n_d_blocks)]
     result_norm_pwz = []
     result_norm_pdz = [[] for i in range(n_d_blocks)]
 
@@ -128,26 +128,79 @@ def plsa_em_step_dask(
     )
     p_z_given_d = da.vstack(p_z_given_d_blocks) / da.hstack(norm_pdz_blocks).T
 
-    return p_w_given_z.compute(), p_z_given_d.compute()
+    result = compute(p_w_given_z, p_z_given_d)
+
+    return result
 
 
 @numba.njit(
     locals={
         "i": numba.types.uint16,
         "j": numba.types.uint16,
-        "k": numba.types.uint16,
+        "k": numba.types.intp,
         "w": numba.types.uint32,
         "d": numba.types.uint32,
         "z": numba.types.uint16,
         "nz_idx": numba.types.uint32,
         "x": numba.types.float32,
-        "result": numba.types.float32,
+        "result": numba.types.float32[:, :, ::1],
         "p_w_given_d": numba.types.float32,
     },
     fastmath=True,
     nogil=True,
     parallel=True,
 )
+def log_likelihood_by_blocks_kernel(
+    block_rows,
+    block_cols,
+    block_vals,
+    p_w_given_z,
+    p_z_given_d,
+    block_row_size,
+    block_col_size,
+    i, j,
+):
+    result = np.zeros((1, 1, 1), dtype=np.float32)
+    k = p_w_given_z.shape[0]
+
+    for nz_idx in range(block_rows.shape[2]):
+        if block_rows[0, 0, nz_idx] < 0:
+            break
+
+        d = block_rows[0, 0, nz_idx] + i * block_row_size
+        w = block_cols[0, 0, nz_idx] + j * block_col_size
+        x = block_vals[0, 0, nz_idx]
+
+        p_w_given_d = 0.0
+        for z in range(k):
+            p_w_given_d += p_w_given_z[z, w] * p_z_given_d[d, z]
+
+        result[0, 0, 0] += x * np.log(p_w_given_d)
+
+    return result
+
+def log_likelihood_by_blocks_kernel_wrapper(
+    block_rows,
+    block_cols,
+    block_vals,
+    p_w_given_z,
+    p_z_given_d,
+    block_row_size,
+    block_col_size,
+    block_info=None,
+):
+    i, j, _ = block_info[0]["chunk-location"]
+    return log_likelihood_by_blocks_kernel(
+        block_rows,
+        block_cols,
+        block_vals,
+        p_w_given_z,
+        p_z_given_d,
+        block_row_size,
+        block_col_size,
+        i, j,
+    )
+
 def log_likelihood_by_blocks(
     block_rows_ndarray,
     block_cols_ndarray,
@@ -157,26 +210,21 @@ def log_likelihood_by_blocks(
     block_row_size,
     block_col_size,
 ):
-    result = 0.0
-    k = p_w_given_z.shape[0]
 
-    for i in numba.prange(block_rows_ndarray.shape[0]):
-        for j in range(block_rows_ndarray.shape[1]):
-            for nz_idx in range(block_rows_ndarray.shape[2]):
-                if block_rows_ndarray[i, j, nz_idx] < 0:
-                    break
+    log_likelihood_per_block = da.map_blocks(
+        log_likelihood_by_blocks_kernel_wrapper,
+        block_rows_ndarray,
+        block_cols_ndarray,
+        block_vals_ndarray,
+        p_w_given_z,
+        p_z_given_d,
+        block_row_size,
+        block_col_size,
+        dtype=np.float32,
+    )
+    result = log_likelihood_per_block.sum()
+    return result.compute()
 
-                d = block_rows_ndarray[i, j, nz_idx] + i * block_row_size
-                w = block_cols_ndarray[i, j, nz_idx] + j * block_col_size
-                x = block_vals_ndarray[i, j, nz_idx]
-
-                p_w_given_d = 0.0
-                for z in range(k):
-                    p_w_given_d += p_w_given_z[z, w] * p_z_given_d[d, z]
-
-                result += x * np.log(p_w_given_d)
-
-    return result
 
 def plsa_fit_inner_dask(
     block_rows_ndarray,
@@ -201,6 +249,10 @@ def plsa_fit_inner_dask(
         block_col_size,
     )
 
+    # block_rows_ndarray, block_cols_ndarray, block_vals_ndarray = persist(
+    #     block_rows_ndarray, block_cols_ndarray, block_vals_ndarray
+    # )
+
     for i in range(n_iter):
         p_w_given_z, p_z_given_d = plsa_em_step_dask(
             block_rows_ndarray,
@@ -230,6 +282,7 @@ def plsa_fit_inner_dask(
 
     return p_z_given_d, p_w_given_z
 
+
 def plsa_fit(
     X,
     k,
@@ -252,8 +305,8 @@ def plsa_fit(
     n = A.shape[0]
     m = A.shape[1]
 
-    block_row_size = np.uint16(np.ceil(A.shape[0] / n_row_blocks))
-    block_col_size = np.uint16(np.ceil(A.shape[1] / n_col_blocks))
+    block_row_size = np.uint32(np.ceil(A.shape[0] / n_row_blocks))
+    block_col_size = np.uint32(np.ceil(A.shape[1] / n_col_blocks))
 
     A_blocks = [[0] * n_col_blocks for i in range(n_row_blocks)]
     max_nnz_per_block = 0
@@ -271,14 +324,16 @@ def plsa_fit(
             if A_blocks[i][j].nnz > max_nnz_per_block:
                 max_nnz_per_block = A_blocks[i][j].nnz
 
+    del A
+
     block_rows_ndarray = np.full(
-        (n_row_blocks, n_col_blocks, max_nnz_per_block), -1, dtype=np.int32
+        (n_row_blocks, n_col_blocks, max_nnz_per_block), -1, dtype=np.int32,
     )
     block_cols_ndarray = np.full(
-        (n_row_blocks, n_col_blocks, max_nnz_per_block), -1, dtype=np.int32
+        (n_row_blocks, n_col_blocks, max_nnz_per_block), -1, dtype=np.int32,
     )
     block_vals_ndarray = np.zeros(
-        (n_row_blocks, n_col_blocks, max_nnz_per_block), dtype=np.float32
+        (n_row_blocks, n_col_blocks, max_nnz_per_block), dtype=np.float32,
     )
     for i in range(n_row_blocks):
         for j in range(n_col_blocks):
@@ -287,6 +342,18 @@ def plsa_fit(
             block_cols_ndarray[i, j, :nnz] = A_blocks[i][j].col
             block_vals_ndarray[i, j, :nnz] = A_blocks[i][j].data
 
+    del A_blocks
+
+    block_rows_ndarray = da.from_array(
+        block_rows_ndarray, chunks=(1, 1, max_nnz_per_block),
+    )
+    block_cols_ndarray = da.from_array(
+        block_cols_ndarray, chunks=(1, 1, max_nnz_per_block),
+    )
+    block_vals_ndarray = da.from_array(
+        block_vals_ndarray, chunks=(1, 1, max_nnz_per_block),
+    )
+
     p_z_given_d, p_w_given_z = plsa_fit_inner_dask(
         block_rows_ndarray,
         block_cols_ndarray,
